@@ -1,7 +1,7 @@
 /*===========================================================================*\
  *                                                                           *
  *                               OpenMesh                                    *
- *      Copyright (C) 2001-2014 by Computer Graphics Group, RWTH Aachen      *
+ *      Copyright (C) 2001-2015 by Computer Graphics Group, RWTH Aachen      *
  *                           www.openmesh.org                                *
  *                                                                           *
  *---------------------------------------------------------------------------* 
@@ -34,8 +34,8 @@
 
 /*===========================================================================*\
  *                                                                           *             
- *   $Revision: 990 $                                                         *
- *   $Date: 2014-02-05 10:01:07 +0100 (Mi, 05 Feb 2014) $                   *
+ *   $Revision: 1198 $                                                         *
+ *   $Date: 2015-01-15 11:34:21 +0100 (Do, 15 Jan 2015) $                   *
  *                                                                           *
 \*===========================================================================*/
 
@@ -137,14 +137,14 @@ void VDPMClientViewerWidget::mesh_coloring()
 
   for (; vIt!=vEnd; ++vIt)
   {
-    node_handle = mesh_.data(vIt.handle()).vhierarchy_node_handle();
+    node_handle = mesh_.data(*vIt).vhierarchy_node_handle();
     ratio = vhierarchy_.node(node_handle).ratio();
     
     r = (unsigned char) ((1.0f - ratio) * myYellow[0] + ratio * myBlue[0]);
     g = (unsigned char) ((1.0f - ratio) * myYellow[1] + ratio * myBlue[1]);
     b = (unsigned char) ((1.0f - ratio) * myYellow[2] + ratio * myBlue[2]);
 
-    mesh_.set_color(vIt.handle(), OpenMesh::Vec3uc(r,g,b));
+    mesh_.set_color(*vIt, OpenMesh::Vec3uc(r,g,b));
   }
 }
 
