@@ -1,7 +1,7 @@
 /*===========================================================================*\
  *                                                                           *
  *                               OpenMesh                                    *
- *      Copyright (C) 2001-2012 by Computer Graphics Group, RWTH Aachen      *
+ *      Copyright (C) 2001-2015 by Computer Graphics Group, RWTH Aachen      *
  *                           www.openmesh.org                                *
  *                                                                           *
  *---------------------------------------------------------------------------* 
@@ -34,8 +34,8 @@
 
 /*===========================================================================*\
  *                                                                           *             
- *   $Revision: 736 $                                                         *
- *   $Date: 2012-10-08 09:30:49 +0200 (Mo, 08. Okt 2012) $                   *
+ *   $Revision: 1198 $                                                         *
+ *   $Date: 2015-01-15 11:34:21 +0100 (Do, 15 Jan 2015) $                   *
  *                                                                           *
 \*===========================================================================*/
 
@@ -275,7 +275,7 @@ MeshViewerWidgetT<M>::draw_openmesh(const std::string& _draw_mode)
      glBegin(GL_TRIANGLES);
      for (; fIt!=fEnd; ++fIt)
      {
-        fvIt = mesh_.cfv_iter(fIt.handle()); 
+        fvIt = mesh_.cfv_iter(*fIt); 
         glVertex3fv( &mesh_.point(fvIt)[0] );
         ++fvIt;
         glVertex3fv( &mesh_.point(fvIt)[0] );
@@ -292,7 +292,7 @@ MeshViewerWidgetT<M>::draw_openmesh(const std::string& _draw_mode)
     {
       glNormal3fv( &mesh_.normal(fIt)[0] );
       
-      fvIt = mesh_.cfv_iter(fIt.handle()); 
+      fvIt = mesh_.cfv_iter(*fIt); 
       glVertex3fv( &mesh_.point(fvIt)[0] );
       ++fvIt;
       glVertex3fv( &mesh_.point(fvIt)[0] );
@@ -325,12 +325,12 @@ MeshViewerWidgetT<M>::draw_openmesh(const std::string& _draw_mode)
     glBegin(GL_TRIANGLES);
     for (; fIt!=fEnd; ++fIt)
     {
-      fvIt = mesh_.cfv_iter(fIt.handle()); 
-      glArrayElement(fvIt.handle().idx());
+      fvIt = mesh_.cfv_iter(*fIt); 
+      glArrayElement(fvIt->idx());
       ++fvIt;
-      glArrayElement(fvIt.handle().idx());
+      glArrayElement(fvIt->idx());
       ++fvIt;
-      glArrayElement(fvIt.handle().idx());
+      glArrayElement(fvIt->idx());
     }
     glEnd();
     
@@ -361,12 +361,12 @@ MeshViewerWidgetT<M>::draw_openmesh(const std::string& _draw_mode)
     glBegin(GL_TRIANGLES);
     for (; fIt!=fEnd; ++fIt)
     {
-      fvIt = mesh_.cfv_iter(fIt.handle()); 
-      glArrayElement(fvIt.handle().idx());
+      fvIt = mesh_.cfv_iter(*fIt); 
+      glArrayElement(fvIt->idx());
       ++fvIt;
-      glArrayElement(fvIt.handle().idx());
+      glArrayElement(fvIt->idx());
       ++fvIt;
-      glArrayElement(fvIt.handle().idx());
+      glArrayElement(fvIt->idx());
     }
     glEnd();
     
@@ -387,14 +387,14 @@ MeshViewerWidgetT<M>::draw_openmesh(const std::string& _draw_mode)
     glBegin(GL_TRIANGLES);
     for (; fIt!=fEnd; ++fIt)
     {
-      glColor( fIt.handle() );
+      glColor( *fIt );
 
-      fvIt = mesh_.cfv_iter(fIt.handle()); 
-      glArrayElement(fvIt.handle().idx());
+      fvIt = mesh_.cfv_iter(*fIt); 
+      glArrayElement(fvIt->idx());
       ++fvIt;
-      glArrayElement(fvIt.handle().idx());
+      glArrayElement(fvIt->idx());
       ++fvIt;
-      glArrayElement(fvIt.handle().idx());
+      glArrayElement(fvIt->idx());
     }
     glEnd();
     
