@@ -1,7 +1,7 @@
 /*===========================================================================*\
  *                                                                           *
  *                               OpenMesh                                    *
- *      Copyright (C) 2001-2012 by Computer Graphics Group, RWTH Aachen      *
+ *      Copyright (C) 2001-2015 by Computer Graphics Group, RWTH Aachen      *
  *                           www.openmesh.org                                *
  *                                                                           *
  *---------------------------------------------------------------------------* 
@@ -34,8 +34,8 @@
 
 /*===========================================================================*\
  *                                                                           *             
- *   $Revision: 866 $                                                         *
- *   $Date: 2013-07-24 10:23:36 +0200 (Mi, 24. Jul 2013) $                   *
+ *   $Revision: 1188 $                                                         *
+ *   $Date: 2015-01-05 16:34:10 +0100 (Mo, 05 Jan 2015) $                   *
  *                                                                           *
 \*===========================================================================*/
 
@@ -57,9 +57,9 @@ PolyConnectivity::find_halfedge(VertexHandle _start_vh, VertexHandle _end_vh ) c
 {
   assert(_start_vh.is_valid() && _end_vh.is_valid());
 
-  for (ConstVertexVertexIter vvIt=cvv_iter(_start_vh); vvIt; ++vvIt)
-    if (vvIt.handle() == _end_vh)
-      return vvIt.current_halfedge_handle();
+  for (ConstVertexOHalfedgeIter voh_it = cvoh_iter(_start_vh); voh_it.is_valid(); ++voh_it)
+    if (to_vertex_handle(*voh_it) == _end_vh)
+      return *voh_it;
 
   return InvalidHalfedgeHandle;
 }
@@ -67,14 +67,14 @@ PolyConnectivity::find_halfedge(VertexHandle _start_vh, VertexHandle _end_vh ) c
 
 bool PolyConnectivity::is_boundary(FaceHandle _fh, bool _check_vertex) const
 {
-  for (ConstFaceEdgeIter cfeit = cfe_iter( _fh ); cfeit; ++cfeit)
-      if (is_boundary( cfeit.handle() ) )
+  for (ConstFaceEdgeIter cfeit = cfe_iter( _fh ); cfeit.is_valid(); ++cfeit)
+      if (is_boundary( *cfeit ) )
         return true;
 
   if (_check_vertex)
   {
-      for (ConstFaceVertexIter cfvit = cfv_iter( _fh ); cfvit; ++cfvit)
-        if (is_boundary( cfvit.handle() ) )
+      for (ConstFaceVertexIter cfvit = cfv_iter( _fh ); cfvit.is_valid(); ++cfvit)
+        if (is_boundary( *cfvit ) )
             return true;
   }
   return false;
@@ -89,9 +89,9 @@ bool PolyConnectivity::is_manifold(VertexHandle _vh) const
     boundary halfedge, the vertex is non-manifold. */
 
   ConstVertexOHalfedgeIter vh_it(*this, _vh);
-  if (vh_it)
-    for (++vh_it; vh_it; ++vh_it)
-        if (is_boundary(vh_it.handle()))
+  if (vh_it.is_valid())
+    for (++vh_it; vh_it.is_valid(); ++vh_it)
+        if (is_boundary(*vh_it))
           return false;
   return true;
 }
@@ -99,11 +99,11 @@ bool PolyConnectivity::is_manifold(VertexHandle _vh) const
 //-----------------------------------------------------------------------------
 void PolyConnectivity::adjust_outgoing_halfedge(VertexHandle _vh)
 {
-  for (ConstVertexOHalfedgeIter vh_it=cvoh_iter(_vh); vh_it; ++vh_it)
+  for (ConstVertexOHalfedgeIter vh_it=cvoh_iter(_vh); vh_it.is_valid(); ++vh_it)
   {
-    if (is_boundary(vh_it.handle()))
+    if (is_boundary(*vh_it))
     {
-      set_halfedge_handle(_vh, vh_it.handle());
+      set_halfedge_handle(_vh, *vh_it);
       break;
     }
   }
@@ -115,7 +115,7 @@ PolyConnectivity::FaceHandle
 PolyConnectivity::add_face(const VertexHandle* _vertex_handles, size_t _vhs_size)
 {
   VertexHandle                   vh;
-  size_t                         i, ii, n(_vhs_size), id;
+  size_t                         i, ii, n(_vhs_size);
   HalfedgeHandle                 inner_next, inner_prev,
                                  outer_next, outer_prev,
                                  boundary_next, boundary_prev,
@@ -191,6 +191,13 @@ PolyConnectivity::add_face(const VertexHandle* _vertex_handles, size_t _vhs_size
         patch_start = next_halfedge_handle(inner_prev);
         patch_end   = prev_halfedge_handle(inner_next);
 
+        assert(boundary_prev.is_valid());
+        assert(patch_start.is_valid());
+        assert(patch_end.is_valid());
+        assert(boundary_next.is_valid());
+        assert(inner_prev.is_valid());
+        assert(inner_next.is_valid());
+
         // relink
         next_cache_[next_cache_count_++] = std::make_pair(boundary_prev, patch_start);
         next_cache_[next_cache_count_++] = std::make_pair(patch_end, boundary_next);
@@ -215,8 +222,10 @@ PolyConnectivity::add_face(const VertexHandle* _vertex_handles, size_t _vhs_size
 
     inner_prev = edgeData_[i].halfedge_handle;
     inner_next = edgeData_[ii].halfedge_handle;
+    assert(inner_prev.is_valid());
+    assert(inner_next.is_valid());
 
-    id = 0;
+    size_t id = 0;
     if (edgeData_[i].is_new)  id |= 1;
     if (edgeData_[ii].is_new) id |= 2;
 
@@ -225,18 +234,22 @@ PolyConnectivity::add_face(const VertexHandle* _vertex_handles, size_t _vhs_size
     {
       outer_prev = opposite_halfedge_handle(inner_next);
       outer_next = opposite_halfedge_handle(inner_prev);
+      assert(outer_prev.is_valid());
+      assert(outer_next.is_valid());
 
       // set outer links
       switch (id)
       {
         case 1: // prev is new, next is old
           boundary_prev = prev_halfedge_handle(inner_next);
+          assert(boundary_prev.is_valid());
           next_cache_[next_cache_count_++] = std::make_pair(boundary_prev, outer_next);
           set_halfedge_handle(vh, outer_next);
           break;
 
         case 2: // next is new, prev is old
           boundary_next = next_halfedge_handle(inner_prev);
+          assert(boundary_next.is_valid());
           next_cache_[next_cache_count_++] = std::make_pair(outer_prev, boundary_next);
           set_halfedge_handle(vh, boundary_next);
           break;
@@ -251,6 +264,8 @@ PolyConnectivity::add_face(const VertexHandle* _vertex_handles, size_t _vhs_size
           {
             boundary_next = halfedge_handle(vh);
             boundary_prev = prev_halfedge_handle(boundary_next);
+            assert(boundary_prev.is_valid());
+            assert(boundary_next.is_valid());
             next_cache_[next_cache_count_++] = std::make_pair(boundary_prev, outer_next);
             next_cache_[next_cache_count_++] = std::make_pair(outer_prev, boundary_next);
           }
@@ -325,10 +340,10 @@ bool PolyConnectivity::is_collapse_ok(HalfedgeHandle v0v1)
     v1v0_triangle = valence(face_handle(v1v0)) == 3;
 
   //in a quadmesh we dont have the "next" or "previous" vhandle, so we need to look at previous and next on both sides
-  VertexHandle v_01_p = from_vertex_handle(prev_halfedge_handle(v0v1));
+  //VertexHandle v_01_p = from_vertex_handle(prev_halfedge_handle(v0v1));
   VertexHandle v_01_n = to_vertex_handle(next_halfedge_handle(v0v1));  
 
-  VertexHandle v_10_p = from_vertex_handle(prev_halfedge_handle(v1v0));
+  //VertexHandle v_10_p = from_vertex_handle(prev_halfedge_handle(v1v0));
   VertexHandle v_10_n = to_vertex_handle(next_halfedge_handle(v1v0));
 
   //are the vertices already deleted ?
@@ -343,12 +358,12 @@ bool PolyConnectivity::is_collapse_ok(HalfedgeHandle v0v1)
   {
     if (v0v1_triangle)
     {
-      VertexHandle vl = to_vertex_handle(next_halfedge_handle(v0v1));
+      //VertexHandle vl = to_vertex_handle(next_halfedge_handle(v0v1));
 
       HalfedgeHandle h1 = next_halfedge_handle(v0v1);
       HalfedgeHandle h2 = next_halfedge_handle(h1);
       if (is_boundary(opposite_halfedge_handle(h1)) && is_boundary(opposite_halfedge_handle(h2)))
-	return false;
+        return false;
     }
   }
 
@@ -358,12 +373,12 @@ bool PolyConnectivity::is_collapse_ok(HalfedgeHandle v0v1)
   {
     if (v1v0_triangle)
     {
-      VertexHandle vr = to_vertex_handle(next_halfedge_handle(v1v0));
+      //VertexHandle vr = to_vertex_handle(next_halfedge_handle(v1v0));
 
       HalfedgeHandle h1 = next_halfedge_handle(v1v0);
       HalfedgeHandle h2 = next_halfedge_handle(h1);
       if (is_boundary(opposite_halfedge_handle(h1)) && is_boundary(opposite_halfedge_handle(h2)))
-	return false;
+        return false;
     }
   }
 
@@ -373,21 +388,21 @@ bool PolyConnectivity::is_collapse_ok(HalfedgeHandle v0v1)
   
   VertexVertexIter vv_it;
   // test intersection of the one-rings of v0 and v1
-  for (vv_it = vv_iter(v0); vv_it; ++vv_it)
+  for (vv_it = vv_iter(v0); vv_it.is_valid(); ++vv_it)
   {
-    status(vv_it).set_tagged(false);
+    status(*vv_it).set_tagged(false);
   }
 
-  for (vv_it = vv_iter(v1); vv_it; ++vv_it)
+  for (vv_it = vv_iter(v1); vv_it.is_valid(); ++vv_it)
   {
-    status(vv_it).set_tagged(true);
+    status(*vv_it).set_tagged(true);
   }
 
-  for (vv_it = vv_iter(v0); vv_it; ++vv_it)
+  for (vv_it = vv_iter(v0); vv_it.is_valid(); ++vv_it)
   {
-    if (status(vv_it).tagged() &&
-      !(vv_it.handle() == v_01_n && v0v1_triangle) &&
-      !(vv_it.handle() == v_10_n && v1v0_triangle)
+    if (status(*vv_it).tagged() &&
+      !(*vv_it == v_01_n && v0v1_triangle) &&
+      !(*vv_it == v_10_n && v1v0_triangle)
       )
     {
       return false;
@@ -425,7 +440,7 @@ bool PolyConnectivity::is_collapse_ok(HalfedgeHandle v0v1)
     }
   }
 
-  if (status(vv_it).tagged() && v_01_n == v_10_n && v0v1_triangle && v1v0_triangle)
+  if (status(*vv_it).tagged() && v_01_n == v_10_n && v0v1_triangle && v1v0_triangle)
   {
     return false;
   }
@@ -441,8 +456,8 @@ void PolyConnectivity::delete_vertex(VertexHandle _vh, bool _delete_isolated_ver
   // store incident faces
   std::vector<FaceHandle> face_handles;
   face_handles.reserve(8);
-  for (VFIter vf_it(vf_iter(_vh)); vf_it; ++vf_it)
-    face_handles.push_back(vf_it.handle());
+  for (VFIter vf_it(vf_iter(_vh)); vf_it.is_valid(); ++vf_it)
+    face_handles.push_back(*vf_it);
 
 
   // delete collected faces
@@ -509,9 +524,9 @@ void PolyConnectivity::delete_face(FaceHandle _fh, bool _delete_isolated_vertice
   //   2) collect all boundary halfedges, set them deleted
   //   3) store vertex handles
   HalfedgeHandle hh;
-  for (FaceHalfedgeIter fh_it(fh_iter(_fh)); fh_it; ++fh_it)
+  for (FaceHalfedgeIter fh_it(fh_iter(_fh)); fh_it.is_valid(); ++fh_it)
   {
-    hh = fh_it.handle();
+    hh = *fh_it;
 
     set_boundary(hh);//set_face_handle(hh, InvalidFaceHandle);
 
@@ -730,8 +745,8 @@ void PolyConnectivity::collapse_edge(HalfedgeHandle _hh)
 
 
   // halfedge -> vertex
-  for (VertexIHalfedgeIter vih_it(vih_iter(vo)); vih_it; ++vih_it)
-    set_vertex_handle(vih_it.handle(), vh);
+  for (VertexIHalfedgeIter vih_it(vih_iter(vo)); vih_it.is_valid(); ++vih_it)
+    set_vertex_handle(*vih_it, vh);
 
 
   // halfedge -> halfedge
@@ -810,7 +825,7 @@ bool PolyConnectivity::is_simple_link(EdgeHandle _eh) const
   HalfedgeHandle heh0 = halfedge_handle(_eh, 0);
   HalfedgeHandle heh1 = halfedge_handle(_eh, 1);
   
-  FaceHandle fh0 = face_handle(heh0);//fh0 or fh1 might be a invalid,
+  //FaceHandle fh0 = face_handle(heh0);//fh0 or fh1 might be a invalid,
   FaceHandle fh1 = face_handle(heh1);//i.e., representing the boundary
   
   HalfedgeHandle next_heh = next_halfedge_handle(heh0);
@@ -829,11 +844,11 @@ bool PolyConnectivity::is_simple_link(EdgeHandle _eh) const
 bool PolyConnectivity::is_simply_connected(FaceHandle _fh) const
 {
   std::set<FaceHandle> nb_fhs;
-  for (ConstFaceFaceIter cff_it = cff_iter(_fh); cff_it; ++cff_it)
+  for (ConstFaceFaceIter cff_it = cff_iter(_fh); cff_it.is_valid(); ++cff_it)
   {
-    if (nb_fhs.find(cff_it) == nb_fhs.end())
+    if (nb_fhs.find(*cff_it) == nb_fhs.end())
     {
-      nb_fhs.insert(cff_it);
+      nb_fhs.insert(*cff_it);
     }
     else
     {//there is more than one link
@@ -895,9 +910,9 @@ PolyConnectivity::remove_edge(EdgeHandle _eh)
   {//rem_fh is the face at heh1
     set_halfedge_handle(rem_fh, prev_heh0);
   }
-  for (FaceHalfedgeIter fh_it = fh_iter(rem_fh); fh_it; ++fh_it)
+  for (FaceHalfedgeIter fh_it = fh_iter(rem_fh); fh_it.is_valid(); ++fh_it)
   {//set the face handle of the halfedges of del_fh to point to rem_fh
-    set_face_handle(fh_it, rem_fh);  
+    set_face_handle(*fh_it, rem_fh);
   } 
   
   status(_eh).set_deleted(true);  
@@ -937,9 +952,9 @@ void PolyConnectivity::reinsert_edge(EdgeHandle _eh)
   set_next_halfedge_handle(prev_heh1, heh1);
   set_prev_halfedge_handle(next_heh1, heh1);
   
-  for (FaceHalfedgeIter fh_it = fh_iter(del_fh); fh_it; ++fh_it)
+  for (FaceHalfedgeIter fh_it = fh_iter(del_fh); fh_it.is_valid(); ++fh_it)
   {//reassign halfedges to del_fh  
-    set_face_handle(fh_it, del_fh);
+    set_face_handle(*fh_it, del_fh);
   }
    
   if (face_handle(halfedge_handle(rem_fh)) == del_fh)
@@ -977,9 +992,9 @@ PolyConnectivity::insert_edge(HalfedgeHandle _prev_heh, HalfedgeHandle _next_heh
   //now set the face handles - the new face is assigned to heh0
   FaceHandle new_fh = new_face();
   set_halfedge_handle(new_fh, heh0);
-  for (FaceHalfedgeIter fh_it = fh_iter(new_fh); fh_it; ++fh_it)
+  for (FaceHalfedgeIter fh_it = fh_iter(new_fh); fh_it.is_valid(); ++fh_it)
   {
-    set_face_handle(fh_it, new_fh);
+    set_face_handle(*fh_it, new_fh);
   }  
   FaceHandle old_fh = face_handle(next_prev_heh);
   set_face_handle(heh1, old_fh);   
@@ -1046,7 +1061,7 @@ void PolyConnectivity::triangulate()
   */
   FaceIter f_it(faces_begin()), f_end(faces_end());
   for (; f_it!=f_end; ++f_it)
-    triangulate(f_it);
+    triangulate(*f_it);
 }
 
 //-----------------------------------------------------------------------------
@@ -1099,15 +1114,15 @@ void PolyConnectivity::split_copy(FaceHandle fh, VertexHandle vh) {
   split(fh, vh);
 
   // Copy the property of the original face to all new faces
-  for(VertexFaceIter vf_it =  vf_iter(vh); vf_it; ++vf_it)
-    copy_all_properties(fh, vf_it);
+  for(VertexFaceIter vf_it =  vf_iter(vh); vf_it.is_valid(); ++vf_it)
+    copy_all_properties(fh, *vf_it);
 }
 
 //-----------------------------------------------------------------------------
 uint PolyConnectivity::valence(VertexHandle _vh) const
 {
   uint count(0);
-  for (ConstVertexVertexIter vv_it=cvv_iter(_vh); vv_it; ++vv_it)
+  for (ConstVertexVertexIter vv_it=cvv_iter(_vh); vv_it.is_valid(); ++vv_it)
     ++count;
   return count;
 }
@@ -1116,7 +1131,7 @@ uint PolyConnectivity::valence(VertexHandle _vh) const
 uint PolyConnectivity::valence(FaceHandle _fh) const
 {
   uint count(0);
-  for (ConstFaceVertexIter fv_it=cfv_iter(_fh); fv_it; ++fv_it)
+  for (ConstFaceVertexIter fv_it=cfv_iter(_fh); fv_it.is_valid(); ++fv_it)
     ++count;
   return count;
 }
@@ -1130,9 +1145,9 @@ void PolyConnectivity::split_edge(EdgeHandle _eh, VertexHandle _vh)
   VertexHandle vfrom = from_vertex_handle(h0);
 
   HalfedgeHandle ph0 = prev_halfedge_handle(h0);
-  HalfedgeHandle ph1 = prev_halfedge_handle(h1);
+  //HalfedgeHandle ph1 = prev_halfedge_handle(h1);
   
-  HalfedgeHandle nh0 = next_halfedge_handle(h0);
+  //HalfedgeHandle nh0 = next_halfedge_handle(h0);
   HalfedgeHandle nh1 = next_halfedge_handle(h1);
   
   bool boundary0 = is_boundary(h0);
